/* Linux Prism II Stumbler - Utility Scan for 802_11 networks under Linux
 * 
 * File : include.h
 * Project : WifiScanner (c) 2002 Herv Schauer Consultants
 * Usage : This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).
 * 
 * Base code was from prismstumbler Jan Fernquist <Jan.B.Fernquist@telia.com>
 * and wlanctl from www.linux-wlan.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/* $Id: include.h,v 1.23 2002/12/26 15:51:40 poggij Exp $ */

#ifndef _WIFISCAN_INCLUDE_H_
#define _WIFISCAN_INCLUDE_H_


#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>

#ifdef STDC_HEADERS
# include <string.h>
#else
# ifndef HAVE_MEMSET
#  error "You didn't have memset function, try to create it or change your OS"
# endif
#endif

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#include <sys/timeb.h>
#include <sys/socket.h>
#include <sys/resource.h>
#include <sys/ioctl.h>
#include <asm/types.h>
#include <linux/netlink.h>
#include <wtap.h>
#include <errno.h>
#include <ctype.h>
#ifdef HAVE_LIBNCURSES
#include <ncurses.h>
#endif
#ifdef LWNG_14
#include <pcap.h>
#endif
/* include from Linux-Wlan-NG */
//#include <wlan/wlan_compat.h>
#include <wlan/version.h>
#include <wlan/p80211types.h>
#include <wlan/p80211msg.h>
#include <wlan/p80211meta.h>
#include <wlan/p80211metamsg.h>
#include <wlan/p80211metastruct.h>
#include <wlan/p80211metadef.h>
#include <wlan/p80211ioctl.h>
#include <wlan/p80211hdr.h>
#include <wlan/p80211mgmt.h>

#ifndef FALSE
#define FALSE (UINT8) 0
#endif
#ifndef TRUE
#define TRUE (UINT8) 1
#endif

/* Version */
// If the fourth number is <> 0 it's a CVS version only
#ifdef LWNG_14
#define WIFISCANNER_VERSION WIFISCANNER_VERSION_NUM " (Wlan driver version >= 0.14)"
#else
#define WIFISCANNER_VERSION WIFISCANNER_VERSION_NUM " (Wlan driver version <= 0.13)"
#endif

/* Different size or constant */
#define MAX_BUFFER_SIZE 4096	/* Size of receive buffer */
#define DEVNAME_LEN 16
#define MCAST_GRP_SNIFF 0x00000002	/* Value used to notify netlink */
#define WLAN_STR_ADDR_LEN 18
#define MAXSIZE_OTHERINFORMATION 0x100
#define HISTORY_SIZE 4
#define MAX_NUMBER_OF_DETECTED_CLIENT 255	/* Maximum of station */

/*
 * How many packet should we read if we have a signal on a channel????
 * After that we force a channel change anyway
 */
#define CHANNEL_READ_COUNT  1

/* We want to do only 28 channel scan per second 
   With this time, the capture was optimized
*/
#define TIME_TO_SLEEP_BEFORE_CHANGE_CHANNEL (1000000/(14*1))	/* in s */
#define TIMEOUT_TRY_TO_READ_PAQUET (1000000/(14*2*CHANNEL_READ_COUNT))	/* s */

/*****************************/
/* MACRO                     */
/*****************************/

/*
Structure of bit field description
---------------------------------
|   |   |   |   |   |   |   |   |
| 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
|   |   |   |   |   |   |   |   |
+---------------+---------------+
| <- SUBTYPE -> | <-  TYPE   -> |

bit set     : do    or is    capable
bit not set : don't or isn't capable

bit 0 = 1 && Dedicated Access point || classic client (CPU + Card)
bit 1 = 1 && Emmit some Beacon || don't Emmit Beacon
bit 2 = 1 && mode P2P (Ad-Hoc) || Not
bit 3 = 1 && Data transmited || Not
bit 4 = 1 && Data go to DS || Not
bit 5 = 1 && Data go from DS || Not
*/

#define TYPE_BIT_AP      0
#define TYPE_BIT_BEACON  1
#define TYPE_BIT_P2P     2
#define TYPE_BIT_DATA    3
#define TYPE_BIT_TODS    4
#define TYPE_BIT_FROMDS  5

#define SET_TYPE_BIT_AP(x)      (x = ((x) | BIT0))
#define SET_TYPE_BIT_BEACON(x)  (x = ((x) | BIT1))
#define SET_TYPE_BIT_P2P(x)     (x = ((x) | BIT2))
#define SET_TYPE_BIT_DATA(x)    (x = ((x) | BIT3))
#define SET_TYPE_BIT_TODS(x)    (x = ((x) | BIT4))
#define SET_TYPE_BIT_FROMDS(x)  (x = ((x) | BIT5))

#define CLEAR_TYPE_BIT_AP(x)      (x =((x) & (~BIT0)))
#define CLEAR_TYPE_BIT_BEACON(x)  (x =((x) & (~BIT1)))
#define CLEAR_TYPE_BIT_P2P(x)     (x =((x) & (~BIT2)))
#define CLEAR_TYPE_BIT_DATA(x)    (x =((x) & (~BIT3)))
#define CLEAR_TYPE_BIT_TODS(x)    (x =((x) & (~BIT4)))
#define CLEAR_TYPE_BIT_FROMDS(x)  (x =((x) & (~BIT5)))

#define GET_TYPE_BIT_AP(x)      (((x) & (~BIT0)) >> TYPE_BIT_AP)
#define GET_TYPE_BIT_BEACON(x)  (((x) & (~BIT1)) >> TYPE_BIT_BEACON)
#define GET_TYPE_BIT_P2P(x)     (((x) & (~BIT2)) >> TYPE_BIT_P2P)
#define GET_TYPE_BIT_DATA(x)    (((x) & (~BIT3)) >> TYPE_BIT_DATA)
#define GET_TYPE_BIT_TODS(x)    (((x) & (~BIT4)) >> TYPE_BIT_TODS)
#define GET_TYPE_BIT_FROMDS(x)  (((x) & (~BIT5)) >> TYPE_BIT_FROMDS)

#define TYPE_BIT_00 "Client"
#define TYPE_BIT_01 "AP Base"
#define TYPE_BIT_02 "AP Base (STA in master mode)"
#define TYPE_BIT_03 "AP Base (dedicated)"
#define TYPE_BIT_04 "Ad-Hoc STA"
#define TYPE_BIT_05 "???5"
#define TYPE_BIT_06 "Ad-Hoc STA (beacon)"
#define TYPE_BIT_07 "Ad-Hoc STA"
#define TYPE_BIT_08 "STA Activity"
#define TYPE_BIT_09 "???9 Reserved"
#define TYPE_BIT_0A "???A Reserved"
#define TYPE_BIT_0B "???B Reserved"
#define TYPE_BIT_0C "???C Reserved"
#define TYPE_BIT_0D "???D Reserved"
#define TYPE_BIT_0E "???E Reserved"
#define TYPE_BIT_0F "???F Reserved"

#define SUBTYPE_BIT_00 "Radio only"
#define SUBTYPE_BIT_01 "Data To DS"
#define SUBTYPE_BIT_02 "Data From DS"
#define SUBTYPE_BIT_03 "Data AP to AP"
#define SUBTYPE_BIT_04 "???4 Reserved"
#define SUBTYPE_BIT_05 "???5 Reserved"
#define SUBTYPE_BIT_06 "???6 Reserved"
#define SUBTYPE_BIT_07 "???7 Reserved"
#define SUBTYPE_BIT_08 "???8 Reserved"
#define SUBTYPE_BIT_09 "???9 Reserved"
#define SUBTYPE_BIT_0A "???A Reserved"
#define SUBTYPE_BIT_0B "???B Reserved"
#define SUBTYPE_BIT_0C "???C Reserved"
#define SUBTYPE_BIT_0D "???D Reserved"
#define SUBTYPE_BIT_0E "???E Reserved"
#define SUBTYPE_BIT_0F "???F Reserved"

#define STR_YES "Yes"
#define STR_NO  "No"

#define STR_WEP    "Wep"
#define STR_NOWEP  "---"

#define STR_STA "STA"
#define STR_AP  "AP"

/*** DEBUG ***/
#define MAX_DEBUG_LEVEL 2

/*
 * HELP of the program
 */
#ifdef LWNG_14
#define HELPTEXT \
"Call with no parameters or with the following options\n" \
"        -F FileName  - Save output to a file as well as stdout\n" \
"        -H Hop       - Number of hops do for rotating channel (default 1)\n" \
"        -S Channel   - Only listen on a specific Channel (1-14)\n" \
"        -V           - Write Version and quit\n" \
"        -W FileName  - Save sniffed data to a file in PCAP format\n" \
"        -D FileName  - Create a file of detected devices, in a .dot format\n" \
"        -d           - Write date in human readable format\n" \
"        -i number    - Number of the interface wlan0 = 0 (default 0)\n" \
"        -M number    - Max packets to capture before exit (0 = unlimited)\n" \
"        -N abcd      - Do not display Ack, Beacon, Control, Data\n" \
"        -v level     - For verbose, level 2 is debugging\n"
#else
#define HELPTEXT \
"Call with no parameters or with the following options\n" \
"        -F FileName  - Save output to a file as well as stdout\n" \
"        -H Hop       - Number of hops do for rotating channel (default 1)\n" \
"        -S Channel   - Only listen on a specific Channel (1-14)\n" \
"        -V           - Write Version and quit\n" \
"        -W FileName  - Save sniffed data to a file in PCAP format\n" \
"        -D FileName  - Create a file of detected devices, in a .dot format\n" \
"        -d           - Write date in human readable format\n" \
"        -i number    - Number of the interface wlan0 = 0 (default 0)\n" \
"        -M number    - Max packets to capture before exit (0 = unlimited)\n" \
"        -N abcd      - Do not display Ack, Beacon, Control, Data\n" \
"        -r           - Do a real-time scan (equvalent to - t 0) \n" \
"        -t usec      - Time to wait before change channel (default 35714)\n" \
"                       Not used in Single channel scan \n" \
"        -v level     - For verbose, level 2 is debugging\n"
#endif


// Function to determine if we must not display some packet
#define IS_DISPLAY_ACK(x)     ( (x > 0) & 1)
#define IS_DISPLAY_BEACON(x)  ( (x > 1) & 1)
#define IS_DISPLAY_CONTROL(x) ( (x > 2) & 1)
#define IS_DISPLAY_DATA(x)    ( (x > 3) & 1)

/*
 * Overlay to directly index varius types of data
 * in Mgmt Frames
 */
typedef struct {
  UINT64 ts;			// TimeStamp
  UINT16 bcn_int;		// Beacon Interval
  UINT16 cap_info;		// Capabilities infos
} FixedField_t;

#define WLAN_SIZEOF_TYPEOFPACKET 6
/* Information that we search */
typedef struct {
  char DestMac[WLAN_STR_ADDR_LEN];
  char SrcMac[WLAN_STR_ADDR_LEN];
  char BssId[WLAN_STR_ADDR_LEN];
  char SSID[WLAN_SSID_MAXLEN + 1];
  UINT8 hasWep;
  UINT8 isAp;
  UINT8 Channel;
  UINT8 Signal;
  UINT8 Noise;
  UINT8 Rate;
  UINT16 TypeOfClient;
  char OtherInformation[MAXSIZE_OTHERINFORMATION];
  char TypeOfPacket[WLAN_SIZEOF_TYPEOFPACKET + 1];
  UINT8 TypeSubtype;
  UINT8 IV[4];
} ScanResult_t;

/* Structure to stock collected data */
typedef struct {
  UINT8 AddMac[WLAN_ADDR_LEN];
  UINT8 BSSID[WLAN_ADDR_LEN];
  char SSID[WLAN_SSID_MAXLEN + 1];
  UINT32 SNMax;			/* = Max of (S/N * 100) */
  UINT32 SNMin;			/* = Min of (S/N * 100) */
  UINT8 RateMax;
  UINT8 RateMin;
  UINT8 hasWep:1;
  UINT8 IsDS:1;			/* if == 1 then this client is in Wire Network (DS) */
  UINT8 IsAP:1;			/* if == 1 then this client is an AP */
  UINT8 IsP2P:1;		/* if == 1 then this client is a part of a P2P connection */
  UINT8 Channel;
  UINT16 bcn_int[HISTORY_SIZE];	/* History of beacon interval (IDS use) */
  UINT64 TimeStamp[HISTORY_SIZE];	/* History of TimeStamp (IDS use) */
  UINT16 SeqNum[HISTORY_SIZE];	/* History of Sequence Number (IDS use) */
  UINT16 SeqNumWithAP[HISTORY_SIZE];	/* History of Sequence Number (IDS use) */
  char OtherInformation[MAXSIZE_OTHERINFORMATION];
  UINT8 LastIV[4];
  float coef_a_TS;		// for analyse Timestamp
  float coef_b_TS;
} ClientInfo_t;

/* Structure to stock Statistics */
typedef struct {
  UINT16 AP;
  UINT16 STA;
  UINT32 Beacon;
  UINT16 SSID;
  UINT16 Channel;
  UINT16 INVLD;
  UINT32 WeakIV;
  UINT32 CryptedPackets;
  UINT32 Packets;
  UINT8 IV[4];
} Statistics_t;

// Capture Frame for Linux-WLAN-NG 
typedef struct CaptureArg_t {
  pcap_t *pcap;
  pcap_dumper_t *dump;
  int offset;
} CaptureArg;

// Capture Frame for Linux-WLAN-NG > 0.1.16
// Take a look at <linux-wlan-ng directory>/doc/capturefrm.txt

#ifdef LWNG_16
typedef struct {
  UINT8 foo;
} CaptureFrm2_Header_t;

typedef struct {
  UINT8 foo;
} CaptureFrm2_Payload_t;

typedef struct {
  p80211_caphdr_t *CaptureHeader;	// AVS capture metadata header
  CaptureFrm2_Header_t *Header;	// 802.11 frame header
  CaptureFrm2_Payload_t *Payload;	// 802.11 frame payload
  UINT32 *FCS;			// 802.11 frame check sequence
} CaptureFrm2_t;
#endif // #ifdef LWNG_16


#endif /* _WIFI_INCLUDE_H_ */

