/* Linux Prism II Stumbler - Utility Scan for 802_11 networks under Linux
 * 
 * File : scanner.c
 * Project : WifiScanner (c) 2002 Herv Schauer Consultants
 * Usage : This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).
 * 
 * Base code was from prismstumbler Jan Fernquist <Jan.B.Fernquist@telia.com>
 * and wlanctl from www.linux-wlan.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: scanner.c,v 1.19 2002/12/26 15:52:05 poggij Exp $
 */

#include <include.h>
#include <src/interface.h>
#include <src/wlan-ng.h>
#include <src/netlink.h>
#include <src/functions.h>
#include <src/analyse.h>
#include <src/scanner.h>
#include <src/crt_io.h>
#include <src/conversion.h>

/*
*/
void HelpAndBye (void);
void VersionAndBye (void);

/*
 * Use globals, ugly ... 
 */
ScanResult_t Res;
UINT8 DebugLevel;
WINDOW *Title_WND, *Panel_WND, *Sum_WND, *RealTime_WND;
Statistics_t Stats;
UINT8 CursesIsInUse = (UINT8) 0;

static UINT8 stop_sniffing = (UINT8) 0;
static UINT8 SingleChannel = (UINT8) 0;

/*
 * Signal Handler
 */
RETSIGTYPE
stop_signal (void)
{
  //fprintf (stderr, "Received CTRL-C - sniffing aborted\n\n");
  stop_sniffing = (UINT8) 1;
}

/*
 * Help
 */
void
HelpAndBye (void)
{
  fprintf (stderr, "%s", HELPTEXT);
  fprintf (stderr, "\nPlease send Bug report to : %s\n\n", WIFISCANNER_BUG);
  exit (EXIT_SUCCESS);
}

/*
 * Write Version only and exit
 */
void
VersionAndBye (void)
{
//  fprintf (stderr, "\nPlease send Bug report to : %s\n\n", WIFISCANNER_BUG);
  exit (EXIT_SUCCESS);
}

/******** MAIN ********/

int
main (int argc, char **argv)
{
  int recvlen, oi;		// Size of packet received
  int MaxPacketPerChannel;	// Number of packet received before the next channel is select
  struct tm *ascii_now = NULL;	// The timestamp in ASCII (human readable)
  struct timeb binary_now;
  char devname[6];		// Device name
  unsigned int TimeToSleepBeforeChangeChannel;	// number of miliseconds before the next channel is select
  UINT8 ChannelHop;		// Channel hop used 
  unsigned char msgbuf[MAX_BUFFER_SIZE];	// packet container
  p80211msg_lnxind_wlansniffrm_t *sniff_info;	// pointer to pure network packet
  struct wtap_pkthdr packet_hdr_info;	// pointer to the header of wtap packet 
  UINT32 CatchPacket = 0, MaxCatchPacket;
  UINT8 DateFormat;
  char PacketStatus[256];
  char *OutFileName = NULL, *OutDumpFileName = NULL, *OutDotFileName = NULL;
  FILE *OutFile = NULL, *OutDumpFile = NULL, *OutDotFile = NULL;
  int wtap_error;
  wtap_dumper *dump_file = NULL;	// pointer to the file used to save pcap data
  UINT8 GotItOne = FALSE;
  UINT8 DoNotDisplay = (UINT8) 0;	// 0000 0000
  //                                      |||\_Ack
  //                                      ||\__Beacon
  //                                      |\___Control
  //                                      \____Data
  UINT8 IDS_is_ON = FALSE;
  UINT8 ids_warning = (UINT8) 0;


  fprintf (stderr, "WifiScanner v%s (c) 2002 Herv Schauer Consultants (",
	   WIFISCANNER_VERSION);
  fprintf (stderr, "Jerome.Poggi@hsc-labs.com)\n");

  /* Check if we are run by UID = 0 */
  if (getuid () != 0) {
    fatal ("Hum hum, you seem to be not have the root capabilities...\n"
	   "Try to obtain root capabilities and re-run me :-)\n");
  }

  /* Initialise defaut params */
  SingleChannel = (UINT8) 0;
  DebugLevel = (UINT8) 0;
  TimeToSleepBeforeChangeChannel = TIME_TO_SLEEP_BEFORE_CHANGE_CHANNEL;
//  TimeToSleepBeforeChangeChannel = 0;
  ChannelHop = (UINT8) 1;
  DateFormat = (UINT8) 1;	// 1 is human readable
  MaxCatchPacket = 0;		// 0 = unlimited
  strcpy (devname, "wlan0");

  /* parse command line */
  ParseCommandLine (argc, argv, &SingleChannel, &OutFileName,
		    &OutDumpFileName, &OutDotFileName, &DebugLevel,
		    &TimeToSleepBeforeChangeChannel, devname, &ChannelHop,
		    &MaxCatchPacket, &DateFormat, &DoNotDisplay, &IDS_is_ON);

  // For information
  debug (1, "Size of ClientInfo_t struct : %6d bytes\n",
	 sizeof (ClientInfo_t));
  debug (1, "Size of ClientInfo_t array  : %6d bytes\n",
	 (sizeof (ClientInfo_t) * MAX_NUMBER_OF_DETECTED_CLIENT));

  // We try to interract with the card, if it's not possible we exit
  if (openCard (devname)) {
    fatal ("The interface can not be found or initialize\n");
  }
  // Sanity check
  if (OutFileName) {
    if ((OutFile = fopen (OutFileName, "a")) == NULL) {
      fatal ("Cant open outfile %s for writing\n", OutFileName);
    }
  }
  // Sanity check
  if (OutDumpFileName) {
    dump_file = wtap_dump_open (OutDumpFileName, WTAP_FILE_PCAP,
				WTAP_ENCAP_IEEE_802_11, 2344, &wtap_error);
    if (!dump_file) {
      fatal ("Cannot make the dump file!\n");
    }
  }

  fprintf (stderr, "Beginning scan of the 802.11b networks...\n");

  (void) setvbuf (stdin, NULL, _IONBF, 0);	// Input go to trash :-)
  (void) setpriority (PRIO_PROCESS, 0, -15);	// take more CPU than normal

  if (signal (SIGINT, (__sighandler_t) stop_signal) == SIG_ERR) {
    fatal ("AAAARRRGGG ! Can't install the signal handler !\n");
  }
  fprintf (stderr, "Use CTRL-C to stop sniffing\n");

#ifdef LWNG_14
  if (openPacket (devname) < 0) {
    fprintf (stderr, "Can't open pcap device\n");
    wtap_dump_close (dump_file, &wtap_error);
    exit (EXIT_FAILURE);
  }
#else
  if (openPacket () < 0) {
    fprintf (stderr, "You usually have to be root to open raw sockets\n");
    wtap_dump_close (dump_file, &wtap_error);
    exit (EXIT_FAILURE);
  }
#endif

#ifdef HAVE_LIBNCURSES
  /* Initialisation of NCURSES */
  CursesIsInUse = TRUE;
  InitScreen ();
  if (IDS_is_ON == TRUE)
    mvwprintw (Sum_WND, 17, 3, "IDS is ON ");
#else
  if (IDS_is_ON == TRUE)
    fprintf (stderr, "IDS is ON");
#endif

  /* Initialisation of the selected channel in single channel scan */
  if (SingleChannel) {
    if (!selectChannel (devname, SingleChannel)) {
//      stop_sniffing = 1;
      debug (0, "Can't change Channel\n");
    }
  }

  while (stop_sniffing == 0) {

    /* Channel change */
    if (!SingleChannel) {
      if (NextChannel (devname, ChannelHop)) {
	stop_sniffing = 1;
	debug (0, "Can't change Channel\n");
	break;
      }
    }

    MaxPacketPerChannel = CHANNEL_READ_COUNT;

    do {
      memset (msgbuf, 0, MAX_BUFFER_SIZE);
      recvlen =
	getPacket (msgbuf, MAX_BUFFER_SIZE, TIMEOUT_TRY_TO_READ_PAQUET);
      /* TIMEOUT_TRY_TO_READ_PAQUET is not used in driver version >= 14 */
      if (recvlen == -2) {
	stop_sniffing = 1;
	break;
      }

      if (recvlen <= 0) {
#ifdef LWNG_14
	/* In version >= 14 of the driver, if no packet is received
	   we can do a sleep, because packet is buffered. So less CPU is used */
	//if (SingleChannel)
	usleep (TIMEOUT_TRY_TO_READ_PAQUET);
#endif
	break;			/* Error or nothing to read */
      }

      Stats.Packets++;
      GotItOne = TRUE;
      memset (&Res, 0, sizeof (Res));

      if (DebugLevel >= 2) {
	int n, i, end;

	// Print the packet number
	// Print an HexDump of the whole packet
	wprintw (RealTime_WND, "Packet number : %ld\n", CatchPacket);
	wprintw (RealTime_WND, "recvlen = %d\nMSG=\n", recvlen);

	for (n = 0; n < recvlen; n += 16) {
	  end = recvlen - n;
	  if (end > 16)
	    end = 16;
	  wprintw (RealTime_WND, "0x%04X - ", n);
	  for (i = 0; i < 16; i++) {
	    /* for don't write outside receved packet */
	    if (i < end)
	      wprintw (RealTime_WND, "%02X", (unsigned int) msgbuf[n + i]);
	    else
	      wprintw (RealTime_WND, "  ");
	    /* Separator every 2 bytes */
	    if (!((i + 1) % 2))
	      wprintw (RealTime_WND, " ");
	  }
	  for (i = 0; i < 16; i++) {
	    // Print caractere if it's printable
	    if (i < end) {
	      if (msgbuf[n + i] >= 0x20 && msgbuf[n + i] <= 0x7E)
		wprintw (RealTime_WND, "%c", (char) msgbuf[n + i]);
	      else
		wprintw (RealTime_WND, ".");
	    }
	  }
	  /* End of line each 16 bytes */
	  wprintw (RealTime_WND, "\n");
	}
	wrefresh (RealTime_WND);
      }
      /* END DEBUG */

 /*** Write DATA to DumpFile ***/
      if (OutDumpFileName) {
	sniff_info = (p80211msg_lnxind_wlansniffrm_t *) msgbuf;

	/*
	 * Setup Wiretap packet header 
	 */
	gettimeofday (&packet_hdr_info.ts, NULL);
	packet_hdr_info.caplen = sniff_info->frmlen.data;
	packet_hdr_info.len = sniff_info->frmlen.data;
	packet_hdr_info.pkt_encap = WTAP_ENCAP_IEEE_802_11;

	oi = sizeof (p80211msg_lnxind_wlansniffrm_t);

	/*
	 * Now we can save the frame to the capture file 
	 */
	if (!wtap_dump (dump_file, &packet_hdr_info, NULL, &msgbuf[oi],
			&wtap_error)) {
	  stop_sniffing = 1;
	  debug (0, "Can't save DATA\n");
	  break;
	}
      }

      /*
       * Analyse Data and Write Result
       */
      if (processPacket (msgbuf, recvlen)) {
	ftime (&binary_now);
	if (DateFormat == 0) {
	  snprintf (PacketStatus, 256,
		    "%d.%03d,\"%s\",%02d,%s,%s,%03d,%03d,%s,%s,%s,%s,%s,%s,%s\n",
		    (int) binary_now.time, binary_now.millitm, Res.SSID,
		    Res.Channel, BoolToWepNoWep (Res.hasWep),
		    BoolToStaAp (Res.isAp), Res.Signal, Res.Noise,
		    Res.DestMac, Res.SrcMac, Res.BssId,
		    RateToString (Res.Rate),
		    TypeOfClientToString (Res.TypeOfClient),
		    SubTypeOfClientToString (Res.TypeOfClient),
		    Res.TypeOfPacket);
	} else {
	  ascii_now = localtime (&binary_now.time);
	  snprintf (PacketStatus, 256,
		    "%02d/%02d/%04d %02d:%02d:%02d.%03d,\"%s\",%02d,%s,%s,%03d,%03d,%s,%s,%s,%s,%s,%s,%s\n",
		    ascii_now->tm_mon, ascii_now->tm_mday,
		    ascii_now->tm_year + 1900, ascii_now->tm_hour,
		    ascii_now->tm_min,
		    (binary_now.millitm ==
		     1000) ? (ascii_now->tm_sec + 1) : ascii_now->tm_sec,
		    (binary_now.millitm == 1000) ? 0 : binary_now.millitm,
		    Res.SSID, Res.Channel, BoolToWepNoWep (Res.hasWep),
		    BoolToStaAp (Res.isAp), Res.Signal, Res.Noise,
		    Res.DestMac, Res.SrcMac, Res.BssId,
		    RateToString (Res.Rate),
		    TypeOfClientToString (Res.TypeOfClient),
		    SubTypeOfClientToString (Res.TypeOfClient),
		    Res.TypeOfPacket);
	}

	// SWITCH : DISPLAY or NOT a sort of Packet
	if (!
	    ((DO_NOT_DISPLAY_ACK) ||
	     (DO_NOT_DISPLAY_BEACON) ||
	     (DO_NOT_DISPLAY_CONTROL) || (DO_NOT_DISPLAY_DATA))) {

#ifdef HAVE_LIBNCURSES
	  wprintw (RealTime_WND, "%s", PacketStatus);
#else
	  fprintf (stdout, "%s", PacketStatus);
#endif
	}
#ifdef HAVE_LIBNCURSES
	WritePanel (TRUE);
	RefreshRealTime_WND (GotItOne);
#endif
	// Write it also on file if wanted
	if (OutFile) {
	  fprintf (OutFile, "%s", PacketStatus);
	  fflush (OutFile);
	}
	// A little mark for better view :)
	debug (1, "--- END OF PACKET PROCESSING ---\n");
	fflush (stdout);
      } else {			/* if (processPacket */
	/* Packet is not analysed */
	// Init of time
	ftime (&binary_now);

	if (Res.TypeOfPacket != NULL) {
	  if (DateFormat == 0) {
	    snprintf (PacketStatus, 256,
		      "%d.%03d,%s\n",
		      (int) binary_now.time, binary_now.millitm,
		      Res.TypeOfPacket);
	  } else {
	    ascii_now = localtime (&binary_now.time);
	    snprintf (PacketStatus, 256,
		      "%02d/%02d/%04d %02d:%02d:%02d.%03d,%s\n",
		      ascii_now->tm_mon, ascii_now->tm_mday,
		      ascii_now->tm_year + 1900, ascii_now->tm_hour,
		      ascii_now->tm_min, ascii_now->tm_sec,
		      binary_now.millitm, Res.TypeOfPacket);
	  }

#ifdef HAVE_LIBNCURSES
	  wprintw (RealTime_WND, "%s", PacketStatus);
	  WritePanel (FALSE);
	  RefreshRealTime_WND (GotItOne);
#else
	  fprintf (stdout, "%s\n", PacketStatus);
#endif
	  // Write it also on file if wanted
	  if (OutFile) {
	    fprintf (OutFile, "%s", PacketStatus);
	  }
	}
      }
      // Exit if we reach the maximum of packets wanted
      ++CatchPacket;
      if (MaxCatchPacket != 0) {
	if (CatchPacket > (MaxCatchPacket + Stats.INVLD)) {
	  stop_sniffing = 1;	// Exit the principal loop
	  MaxPacketPerChannel = 1;	// Exit the little loop
	  printf ("Max packets reached\n");
	}
      }
    }
    while (--MaxPacketPerChannel > 0);	/* Until we read enough */

    if (GotItOne) {
      if (IDS_is_ON == TRUE)
	ids_warning = IDS ();
      if (ids_warning != 0) {
	ascii_now = localtime (&binary_now.time);
#ifdef HAVE_LIBNCURSES
	if (DateFormat == 0) {
	  mvwprintw (Panel_WND, ROW_WND_PANEL - 1, 24, "IDS WARNING (%d)",
		     (int) binary_now.time);
	} else {
	  mvwprintw (Panel_WND, ROW_WND_PANEL - 1, 24,
		     "IDS WARNING (%02d/%02d/%04d %02d:%02d:%02d)",
		     ascii_now->tm_mon, ascii_now->tm_mday,
		     ascii_now->tm_year + 1900, ascii_now->tm_hour,
		     ascii_now->tm_min, ascii_now->tm_sec);
	}
#else
	if (DateFormat == 0) {

	  fprintf (stderr, "IDS WARNING (%d)", (int) binary_now.time);
	} else {
	  fprintf (stderr, "IDS WARNING (%02d/%02d/%04d %02d:%02d:%02d)",
		   ascii_now->tm_mon, ascii_now->tm_mday,
		   ascii_now->tm_year + 1900, ascii_now->tm_hour,
		   ascii_now->tm_min, ascii_now->tm_sec);
	}
#endif
      }
    }
#ifdef HAVE_LIBNCURSES
    WriteSummary ();
    RefreshRealTime_WND (GotItOne);
    // For refreshing screen only one time per second
    GotItOne = FALSE;
#endif
    //doupdate();
#ifdef LWNG_13
    // a little sleep, for don't use too many CPU,
    // I discover, that a lot of time, it help to had a better result in the rechearch
    if ((!SingleChannel) && (TimeToSleepBeforeChangeChannel != 0)) {
      usleep (TimeToSleepBeforeChangeChannel);
    }
#endif
  }

  /* while still sniffing */
  /* End of NCURSES */
  EndCurses ();

  if (OutFile) {		// Close file if needed, it's better clean
    fclose (OutFile);
  }

  if (OutDumpFile) {		// Close file if needed, it's better clean
    wtap_dump_close (dump_file, &wtap_error);
  }

  closePacket ();		// Close RaxSocket
  shutCard (devname);		// put card in a normal state

  LogWriteReport ();		// Write the summary

  if (OutDotFileName) {		// And the .dot file also
    if ((OutDotFile = fopen (OutDotFileName, "w")) == NULL) {
      fprintf (stderr, "Cant open outfile %s for writing\n", OutDotFileName);
      exit (EXIT_FAILURE);
    }
    LogWriteDOT (OutDotFile);
    fclose (OutDotFile);
  }

  exit (EXIT_SUCCESS);
}
